@@ -50,31 +50,6 @@
             ]).
 
 
-                 /*******************************
-                 *       FILE_SEARCH_PATH       *
-                 *******************************/
-
-:- multifile user:file_search_path/2.
-
-user:file_search_path(app_data, PrologAppData) :-
-    (   current_prolog_flag(windows, true)
-    ->  catch(win_folder(appdata, AppData), _, fail),
-        atom_concat(AppData, '/SWI-Prolog', PrologAppData),
-        (   exists_directory(PrologAppData)
-        ->  true
-        ;   catch(make_directory(PrologAppData), _, fail)
-        )
-    ;   catch(expand_file_name('~/lib/swipl', [PrologAppData]), _, fail)
-    ).
-user:file_search_path(app_preferences, Preferences) :-
-    (   current_prolog_flag(windows, true)
-    ->  Preferences = app_data('.')
-    ;   catch(expand_file_name(~, [UserHome]), _, fail)
-    ->  Preferences = UserHome
-    ).
-user:file_search_path(user_profile, app_preferences('.')).
-
-
                  /*******************************
                  *         VERSION BANNER       *
                  *******************************/
@@ -126,7 +101,7 @@ version(Message) :-
     !,
     ensure_loaded(user:InitFile).
 '$load_init_file'(Base) :-
-    absolute_file_name(user_profile(Base), InitFile,
+    absolute_file_name(user_app_config(Base), InitFile,
                        [ access(read),
                          file_errors(fail)
                        ]),
@@ -134,6 +109,17 @@ version(Message) :-
     load_files(user:InitFile,
                [ scope_settings(false)
                ]).
+'$load_init_file'('init.pl') :-
+    (   current_prolog_flag(windows, true),
+        absolute_file_name(user_profile('swipl.ini'), InitFile,
+                           [ access(read),
+                             file_errors(fail)
+                           ])
+    ;   expand_file_name('~/.swiplrc', [InitFile]),
+        exists_file(InitFile)
+    ),
+    !,
+    print_message(warning, backcomp(init_file_moved(InitFile))).
 '$load_init_file'(_).
 
 '$load_system_init_file' :-
