@@ -416,7 +416,7 @@ process_use_module2(File, Imports, Src) :-
 prolog_colourise_query(QueryString, SourceID, ColourItem) :-
     query_colour_state(SourceID, ColourItem, TB),
     setup_call_cleanup(
-        save_settings(TB, State),
+        save_settings(TB, [], State),
         colourise_query(QueryString, TB),
         restore_settings(State)).
 
@@ -1920,6 +1920,7 @@ colourise_decl_options(_, Which, TB, Pos) :-
 valid_decl_option(subsumptive,   table).
 valid_decl_option(variant,       table).
 valid_decl_option(incremental,   table).
+valid_decl_option(opaque,        table).
 valid_decl_option(incremental,   dynamic).
 valid_decl_option(abstract(0),   dynamic).
 valid_decl_option(shared,        table).
