@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2017, VU University Amsterdam
+    Copyright (c)  2011-2019, VU University Amsterdam
+                              CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -62,7 +63,7 @@ terminal and the system supports history.
 %   are stored.
 
 history_directory(Dir) :-
-    absolute_file_name(app_config('dir-history'),
+    absolute_file_name(user_app_config('dir-history'),
                        Dir,
                        [ access(write),
                          file_type(directory),
@@ -70,16 +71,18 @@ history_directory(Dir) :-
                        ]),
     !.
 history_directory(Dir) :-
-    absolute_file_name(app_config('.'),
-                       Home,
+    absolute_file_name(user_app_config('.'),
+                       ConfigDir,
                        [ access(write),
                          file_type(directory),
                          file_errors(fail)
                        ]),
-    atom_concat(Home, '/dir-history', Dir),
+    atom_concat(ConfigDir, '/dir-history', Dir),
     (   exists_directory(Dir)
-    ->  fail
-    ;   make_directory(Dir)
+    ->  '$my_file'(Dir)
+    ;   file_directory_name(Dir, Parent),
+        '$my_file'(Parent),
+        make_directory(Dir)
     ).
 
 %!  dir_history_file(+Dir, -File) is det.
