@@ -315,6 +315,18 @@ CpuTime(cputime_kind which)
 #endif /*__WINDOWS__*/
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+clock_gettime() is provided by MinGW32,  but   where  time_t is 64 bits,
+only a 32-bit value is currectly   filled, making get_time/1 return very
+large bogus values. Ideally this should have   a  runtime check. For now
+we'll  hope  that  32-bit  Windows  is   extinct  before  32-bit  time_t
+overflows.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+#ifdef WIN32
+#undef HAVE_CLOCK_GETTIME
+#endif
+
 double
 WallTime(void)
 { double stime;
@@ -982,62 +994,125 @@ initExpand(void)
 }
 
 #ifdef O_CANONICALISE_DIRS
+#define OS_DIR_TABLE_SIZE 32
+
+static unsigned int
+dir_key(const char *name, unsigned int size)
+{ unsigned int k = MurmurHashAligned2(name, strlen(name), MURMUR_SEED);
+
+  return k & (size-1);
+}
+
+static CanonicalDir
+lookupCanonicalDir(const char *name)
+{ if ( GD->os.dir_table.size )
+  { CanonicalDir cd;
+    unsigned int k = dir_key(name, GD->os.dir_table.size);
+
+    for(cd = GD->os.dir_table.entries[k]; cd; cd = cd->next)
+    { if ( streq(cd->name, name) )
+	return cd;
+    }
+  }
+
+  return NULL;
+}
+
+
+static CanonicalDir
+lookupCanonicalDirFromId(const statstruct *buf)
+{ if ( GD->os.dir_table.size )
+  { unsigned i;
+
+    for(i=0; i<GD->os.dir_table.size; i++)
+    { CanonicalDir dn = GD->os.dir_table.entries[i];
+
+      for( ; dn; dn = dn->next )
+      { if ( dn->inode  == buf->st_ino &&
+	     dn->device == buf->st_dev )
+	  return dn;
+      }
+    }
+  }
+
+  return NULL;
+}
+
+
+static CanonicalDir
+createCanonicalDir(const char *name, const char *canonical, const statstruct *buf)
+{ CanonicalDir cd;
+
+  if ( !GD->os.dir_table.entries )
+  { size_t bytes = sizeof(*GD->os.dir_table.entries)*OS_DIR_TABLE_SIZE;
+
+    GD->os.dir_table.entries = PL_malloc(bytes);
+    memset(GD->os.dir_table.entries, 0, bytes);
+    GD->os.dir_table.size = OS_DIR_TABLE_SIZE;
+  }
+
+  unsigned int k = dir_key(name, GD->os.dir_table.size);
+  cd = PL_malloc(sizeof(*cd));
+  cd->name      = store_string(name);
+  cd->canonical = name == canonical ? cd->name : store_string(canonical);
+  cd->device    = buf->st_dev;
+  cd->inode     = buf->st_ino;
+
+  cd->next = GD->os.dir_table.entries[k];
+  GD->os.dir_table.entries[k] = cd;
+
+  return cd;
+}
+
 
 static void
-cleanupExpand(void)
-{ CanonicalDir dn = canonical_dirlist, next;
+deleteCanonicalDir(CanonicalDir d)
+{ unsigned int k = dir_key(d->name, GD->os.dir_table.size);
 
-  canonical_dirlist = NULL;
-  for( ; dn; dn = next )
-  { next = dn->next;
-    if ( dn->canonical && dn->canonical != dn->name )
-      remove_string(dn->canonical);
-    remove_string(dn->name);
-    PL_free(dn);
-  }
-  if ( GD->paths.CWDdir )
-  { remove_string(GD->paths.CWDdir);
-    GD->paths.CWDdir = NULL;
-    GD->paths.CWDlen = 0;
+  if ( d == GD->os.dir_table.entries[k] )
+  { GD->os.dir_table.entries[k] = d->next;
+  } else
+  { CanonicalDir cd;
+
+    for(cd=GD->os.dir_table.entries[k]; cd; cd=cd->next)
+    { if ( cd->next == d )
+      { cd->next = d->next;
+
+	remove_string(d->name);
+	if ( d->canonical != d->name )
+	  remove_string(d->canonical);
+	PL_free(d);
+      }
+    }
+
+    assert(0);
   }
 }
 
 
 static void
-registerParentDirs(const char *path)
-{ const char *e = path + strlen(path);
+cleanupExpand(void)
+{ if ( GD->os.dir_table.size )
+  { unsigned i;
 
-  while(e>path)
-  { char dirname[MAXPATHLEN];
-    char tmp[MAXPATHLEN];
-    CanonicalDir d;
-    statstruct buf;
+    for(i=0; i<GD->os.dir_table.size; i++)
+    { CanonicalDir dn = GD->os.dir_table.entries[i];
+      CanonicalDir next;
 
-    for(e--; *e != '/' && e > path + 1; e-- )
-      ;
-
-    strncpy(dirname, path, e-path);
-    dirname[e-path] = EOS;
-
-    for(d = canonical_dirlist; d; d = d->next)
-    { if ( streq(d->name, dirname) )
-	return;
+      for( ; dn; dn = next )
+      { next = dn->next;
+	if ( dn->canonical && dn->canonical != dn->name )
+	  remove_string(dn->canonical);
+	remove_string(dn->name);
+	PL_free(dn);
+      }
     }
 
-    if ( statfunc(OsPath(dirname, tmp), &buf) == 0 )
-    { CanonicalDir dn   = PL_malloc(sizeof(*dn));
-
-      dn->name		= store_string(dirname);
-      dn->inode		= buf.st_ino;
-      dn->device	= buf.st_dev;
-      dn->canonical	= dn->name;
-      dn->next		= canonical_dirlist;
-      canonical_dirlist	= dn;
-
-      DEBUG(1, Sdprintf("Registered canonical dir %s\n", dirname));
-    } else
-      return;
+    GD->os.dir_table.size = 0;
+    PL_free(GD->os.dir_table.entries);
   }
+
+  PL_changed_cwd();
 }
 
 
@@ -1056,31 +1131,15 @@ verify_entry(CanonicalDir d)
 	 d->device == buf.st_dev )
       return TRUE;
 
-    DEBUG(1, Sdprintf("%s: inode/device changed\n", d->canonical));
+    DEBUG(MSG_OS_DIR, Sdprintf("%s: inode/device changed\n", d->canonical));
 
     d->inode  = buf.st_ino;
     d->device = buf.st_dev;
     return TRUE;
   } else
-  { DEBUG(1, Sdprintf("%s: no longer exists\n", d->canonical));
+  { DEBUG(MSG_OS_DIR, Sdprintf("%s: no longer exists\n", d->canonical));
 
-    if ( d == canonical_dirlist )
-    { canonical_dirlist = d->next;
-    } else
-    { CanonicalDir cd;
-
-      for(cd=canonical_dirlist; cd; cd=cd->next)
-      { if ( cd->next == d )
-	{ cd->next = d->next;
-	  break;
-	}
-      }
-    }
-
-    remove_string(d->name);
-    if ( d->canonical != d->name )
-      remove_string(d->canonical);
-    PL_free(d);
+    deleteCanonicalDir(d);
   }
 
   return FALSE;
@@ -1088,85 +1147,67 @@ verify_entry(CanonicalDir d)
 
 
 static char *
-canonicaliseDir(char *path)
-{ CanonicalDir d, next;
+canonicaliseDir_sync(char *path)
+{ CanonicalDir d;
   statstruct buf;
   char tmp[MAXPATHLEN];
 
-  DEBUG(1, Sdprintf("canonicaliseDir(%s) --> ", path));
+  DEBUG(MSG_OS_DIR, Sdprintf("canonicaliseDir(%s) --> ", path));
 
-  for(d = canonical_dirlist; d; d = next)
-  { next = d->next;
+  if ( (d=lookupCanonicalDir(path)) && verify_entry(d) )
+  { if ( d->name != d->canonical )
+      strcpy(path, d->canonical);
 
-    if ( streq(d->name, path) && verify_entry(d) )
-    { if ( d->name != d->canonical )
-	strcpy(path, d->canonical);
+    DEBUG(MSG_OS_DIR, Sdprintf("(lookup ino=%ld) %s\n", (long)d->inode, path));
+    return path;
+  }
 
-      DEBUG(1, Sdprintf("(lookup) %s\n", path));
+  if ( statfunc(OsPath(path, tmp), &buf) == 0 )
+  { char parent[MAXPATHLEN];
+    char *e = path + strlen(path);
+
+    DEBUG(MSG_OS_DIR, Sdprintf("Looking for ino=%ld\n", buf.st_ino));
+    if ( (d=lookupCanonicalDirFromId(&buf)) &&
+	 verify_entry(d) )
+    { DEBUG(MSG_OS_DIR, Sdprintf("(found by id)\n"));
+      strcpy(path, d->canonical);
+      return path;
+    }
+
+    for(e--; *e != '/' && e > path + 1; e-- )
+      ;
+    if ( e > path )
+    { strncpy(parent, path, e-path);
+      parent[e-path] = EOS;
+
+      canonicaliseDir_sync(parent);
+      strcpy(parent+strlen(parent), e);
+
+      createCanonicalDir(path, parent, &buf);
+      strcpy(path, parent);
+      DEBUG(MSG_OS_DIR, Sdprintf("(new ino=%ld) %s\n", (long)buf.st_ino, path));
+      return path;
+    } else
+    { createCanonicalDir(path, path, &buf);
       return path;
     }
   }
 
-					/* we need to use malloc() here */
-					/* because allocHeapOrHalt() only ensures */
-					/* alignment for `word', and inode_t */
-					/* is sometimes bigger! */
-
-  if ( statfunc(OsPath(path, tmp), &buf) == 0 )
-  { CanonicalDir dn = PL_malloc(sizeof(*dn));
-    char dirname[MAXPATHLEN];
-    char *e = path + strlen(path);
-
-    dn->name   = store_string(path);
-    dn->inode  = buf.st_ino;
-    dn->device = buf.st_dev;
-
-    do
-    { strncpy(dirname, path, e-path);
-      dirname[e-path] = EOS;
-      if ( statfunc(OsPath(dirname, tmp), &buf) < 0 )
-	break;
-
-      DEBUG(2, Sdprintf("Checking %s (dev=%d,ino=%d)\n",
-			dirname, buf.st_dev, buf.st_ino));
-
-      for(d = canonical_dirlist; d; d = next)
-      { next = d->next;
-
-	if ( d->inode == buf.st_ino && d->device == buf.st_dev &&
-	     verify_entry(d) )
-	{ DEBUG(2, Sdprintf("Hit with %s (dev=%d,ino=%d)\n",
-			    d->canonical, d->device, d->inode));
-
-	  strcpy(dirname, d->canonical);
-	  strcat(dirname, e);
-	  strcpy(path, dirname);
-	  dn->canonical = store_string(path);
-	  dn->next = canonical_dirlist;
-	  canonical_dirlist = dn;
-	  DEBUG(1, Sdprintf("(replace) %s\n", path));
-	  registerParentDirs(path);
-	  return path;
-	}
-      }
-
-      for(e--; *e != '/' && e > path + 1; e-- )
-	;
-    } while( e > path );
-
-    dn->canonical = dn->name;
-    dn->next = canonical_dirlist;
-    canonical_dirlist = dn;
-
-    DEBUG(1, Sdprintf("(new, existing) %s\n", path));
-    registerParentDirs(path);
-    return path;
-  }
-
-  DEBUG(1, Sdprintf("(nonexisting) %s\n", path));
+  DEBUG(MSG_OS_DIR, Sdprintf("(nonexisting) %s\n", path));
   return path;
 }
 
+static char *
+canonicaliseDir(char *path)
+{ char *s;
+
+  PL_LOCK(L_OSDIR);
+  s = canonicaliseDir_sync(path);
+  PL_UNLOCK(L_OSDIR);
+
+  return s;
+}
+
 #else
 
 #define canonicaliseDir(d)
@@ -1602,6 +1643,7 @@ AbsoluteFile(const char *spec, char *path)
   char tmp[MAXPATHLEN];
   char buf[MAXPATHLEN];
   char *file = PrologPath(spec, buf, sizeof(buf));
+  size_t cwdlen;
 
   if ( !file )
     return (char *) NULL;
@@ -1631,14 +1673,14 @@ AbsoluteFile(const char *spec, char *path)
 
   if ( !PL_cwd(path, MAXPATHLEN) )
     return NULL;
+  cwdlen = strlen(path);
 
-  if ( (GD->paths.CWDlen + strlen(file) + 1) >= MAXPATHLEN )
+  if ( (cwdlen + strlen(file) + 1) >= MAXPATHLEN )
   { PL_error(NULL, 0, NULL, ERR_REPRESENTATION, ATOM_max_path_length);
     return (char *) NULL;
   }
 
-  strcpy(path, GD->paths.CWDdir);
-  strcpy(&path[GD->paths.CWDlen], file);
+  strcpy(&path[cwdlen], file);
 
   return canonicalisePath(path);
 }
@@ -1805,6 +1847,18 @@ DirName(const char *f, char *dir)
     on `path'.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+static int
+is_cwd(const char *dir)
+{ int rc;
+
+  PL_LOCK(L_OS);
+  rc = (GD->paths.CWDdir && streq(dir, GD->paths.CWDdir));
+  PL_UNLOCK(L_OS);
+
+  return rc;
+}
+
+
 bool
 ChDir(const char *path)
 { char ospath[MAXPATHLEN];
@@ -1812,12 +1866,13 @@ ChDir(const char *path)
 
   OsPath(path, ospath);
 
-  if ( path[0] == EOS || streq(path, ".") ||
-       (GD->paths.CWDdir && streq(path, GD->paths.CWDdir)) )
+  if ( path[0] == EOS || streq(path, ".") || is_cwd(path) )
     return TRUE;
 
   if ( !AbsoluteFile(path, tmp) )
     return FALSE;
+  if ( is_cwd(tmp) )
+    return TRUE;
 
   if ( chdir(ospath) == 0 )
   { size_t len;
@@ -1827,7 +1882,7 @@ ChDir(const char *path)
     { tmp[len++] = '/';
       tmp[len] = EOS;
     }
-    PL_LOCK(L_OS);					/* Lock with PL_changed_cwd() */
+    PL_LOCK(L_OS);				/* Lock with PL_changed_cwd() */
     GD->paths.CWDlen = len;			/* and PL_cwd() */
     if ( GD->paths.CWDdir )
       remove_string(GD->paths.CWDdir);
